# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 TU Wien.
#

import pathlib
import re
import tempfile

from typing import Optional

from inveniordm_py.client import InvenioAPI
from inveniordm_py.files.metadata import FilesListMetadata, OutgoingStream
from inveniordm_py.records.metadata import DraftMetadata
from inveniordm_py.records.resources import Draft


doi_re = re.compile(r"^((https?://)?doi.org/)?(10\.\d+)/(.*)$")
url_re = re.compile(r"^.*(/api)?/records/(.*)$")


class InvenioRDM:
    """Utility class for connecting to an InvenioRDM instance."""

    DEFAULT_URL = "https://researchdata.tuwien.ac.at"

    def __init__(self, api_token: str, url: str = None):
        """InvenioRDM constructor."""
        self.api_token = api_token
        self.url = (url or self.DEFAULT_URL).rstrip("/")

        # Ensure the URL contains /api as the InvenioAPI client
        # can only fetch JSON
        if not self.url.endswith("/api"):
            self.url += "/api"

        self.client = InvenioAPI(base_url=self.url, access_token=self.api_token)

    def _normalize_pid(self, record_pid: str) -> str:
        """Normalize the given record PID.

        If the input is either a DOI or a URL pointing to the record's landing page
        (or API endpoint), the persistent identifier will be extracted from the string.
        Otherwise, the input will be assumed to be a persistent identifier already,
        and returned as is.
        """

        if match := doi_re.match(record_pid):
            # the last part of the DOI
            return match.group(4)

        elif record_pid.startswith(self.url) and (match := url_re.match(record_pid)):
            # the part in the URL after "/records/"
            return match.group(2)

        return record_pid

    def _resolve_draft(self, record_pid: str) -> Draft:
        """Returns a draft object, given a record PID."""
        record = self.client.records(self._normalize_pid(record_pid))
        return record.draft

    def create(self, metadata: dict) -> str:
        """Creates the record with the given metadata."""
        draft = self.client.records.create(data=DraftMetadata(**metadata))
        return draft.data["id"]

    def update(self, record_pid: str, metadata: dict) -> bool:
        """Updates the draft metadata."""
        draft = self._resolve_draft(record_pid)

        try:
            draft.get()
            draft.update(data=DraftMetadata(**metadata))
            return True

        except Exception as e:
            print(f"Draft update failed: {e}")
            return False

    def upload(self, record_pid: str, file_name: str, file_path: str) -> Optional[str]:
        """Uploads a file to the specified draft."""
        draft = self._resolve_draft(record_pid)

        try:
            draft.get()
            file_meta = FilesListMetadata([{"key": file_name}])
            file = draft.files.create(file_meta)

            stream = open(file_path, "rb")
            for f in draft.files:
                f.set_contents(OutgoingStream(data=stream))
                f.commit()

            return file.data["entries"][0]["key"]

        except Exception as e:
            print(f"File upload failed: {e}")

    def remove(self, record_pid: str, file_name: str) -> bool:
        """Removes given file from the specified draft."""
        draft = self._resolve_draft(record_pid)

        try:
            draft.get()
            draft.files(file_name).delete()
            return True

        except Exception as e:
            print(f"File removal failed: {e}")
            return False

    def download(self, record_pid: str, file_name: str) -> Optional[str]:
        """Downloads given file from the specified (published) record."""
        record = self.client.records(self._normalize_pid(record_pid))

        try:
            record.get()
            response = record.files(file_name).download()

            with tempfile.NamedTemporaryFile(delete=False) as downloaded_file:

                for chunk in response.iter_content(chunk_size=256):
                    downloaded_file.write(chunk)

            file_path = pathlib.Path(downloaded_file.name)
            return str(file_path)

        except Exception as e:
            print(f"File download failed: {e}")

    def publish(self, record_pid: str) -> Optional[str]:
        """Publishes draft given draft."""
        draft = self._resolve_draft(record_pid)

        try:
            record = draft.publish()
            return record.data["id"]

        except Exception as e:
            print(f"Draft publish failed: {e}")
